# -*- coding: utf-8 -*-
from setuphelpers import *
import time

# Update for Removal of Adobe Flash Player for Windows (KB4577586)

""" 
Sources: 
https://helpx.adobe.com/flash-player/kb/uninstall-flash-player-windows.html
https://blogs.windows.com/msedgedev/2020/09/04/update-adobe-flash-end-support/
https://www.catalog.update.microsoft.com/search.aspx?q=4577586

Infos :
{44CF4DB9-707A-4395-839C-573FBC206CB9} Adobe Flash Player 19 ActiveX    19.0.0.245  MsiExec.exe /X{44CF4DB9-707A-4395-839C-573FBC206CB9}
"""


# Defining variables
kbs_dict = [
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_c9bb49df325c20ddc147137b3f769fa44d800dde.msu",
        "kb_name": "windows10.0-kb4577586-x64_c9bb49df325c20ddc147137b3f769fa44d800dde.msu",
        "win_vers": WindowsVersions.Windows10v20H2,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_238da6b730a3f6a709f31980b87e2d8ecb0a8973.msu",
        "kb_name": "windows10.0-kb4577586-x86_238da6b730a3f6a709f31980b87e2d8ecb0a8973.msu",
        "win_vers": WindowsVersions.Windows10v20H2,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_c9bb49df325c20ddc147137b3f769fa44d800dde.msu",
        "kb_name": "windows10.0-kb4577586-x64_c9bb49df325c20ddc147137b3f769fa44d800dde.msu",
        "win_vers": WindowsVersions.Windows10v2004,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_238da6b730a3f6a709f31980b87e2d8ecb0a8973.msu",
        "kb_name": "windows10.0-kb4577586-x86_238da6b730a3f6a709f31980b87e2d8ecb0a8973.msu",
        "win_vers": WindowsVersions.Windows10v2004,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_ec16e118cd8b99df185402c7a0c65a31e031a6f0.msu",
        "kb_name": "windows10.0-kb4577586-x64_ec16e118cd8b99df185402c7a0c65a31e031a6f0.msu",
        "win_vers": WindowsVersions.Windows10v1909,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_002b1ffa7fb31836aeaed74007a6949d31a96460.msu",
        "kb_name": "windows10.0-kb4577586-x86_002b1ffa7fb31836aeaed74007a6949d31a96460.msu",
        "win_vers": WindowsVersions.Windows10v1909,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_ec16e118cd8b99df185402c7a0c65a31e031a6f0.msu",
        "kb_name": "windows10.0-kb4577586-x64_ec16e118cd8b99df185402c7a0c65a31e031a6f0.msu",
        "win_vers": WindowsVersions.Windows10v1903,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_002b1ffa7fb31836aeaed74007a6949d31a96460.msu",
        "kb_name": "windows10.0-kb4577586-x86_002b1ffa7fb31836aeaed74007a6949d31a96460.msu",
        "win_vers": WindowsVersions.Windows10v1903,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_d0f434327db9e3308b86591c248c825c03687632.msu",
        "kb_name": "windows10.0-kb4577586-x64_d0f434327db9e3308b86591c248c825c03687632.msu",
        "win_vers": WindowsVersions.Windows10v1809,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_947f151a8bb5a0b0797702e53431c52d56c02c32.msu",
        "kb_name": "windows10.0-kb4577586-x86_947f151a8bb5a0b0797702e53431c52d56c02c32.msu",
        "win_vers": WindowsVersions.Windows10v1809,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_07eba1db5a2cfac83683bd43d727a3195a94b39b.msu",
        "kb_name": "windows10.0-kb4577586-x64_07eba1db5a2cfac83683bd43d727a3195a94b39b.msu",
        "win_vers": WindowsVersions.Windows10v1803,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_b737ffa841d2aab5cbf5e00560b93577a6d0b4d2.msu",
        "kb_name": "windows10.0-kb4577586-x86_b737ffa841d2aab5cbf5e00560b93577a6d0b4d2.msu",
        "win_vers": WindowsVersions.Windows10v1803,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_e0921a984710c3992b68e570115b29b9e8e5c708.msu",
        "kb_name": "windows10.0-kb4577586-x64_e0921a984710c3992b68e570115b29b9e8e5c708.msu",
        "win_vers": WindowsVersions.Windows10v1709,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_f1ee6a8c9cdc91b4486fde9dc98c9dc5f9babc77.msu",
        "kb_name": "windows10.0-kb4577586-x86_f1ee6a8c9cdc91b4486fde9dc98c9dc5f9babc77.msu",
        "win_vers": WindowsVersions.Windows10v1709,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x64_84ef2e9e7eb6bd2a3cfa5a8aef59adccab988e7d.msu",
        "kb_name": "windows10.0-kb4577586-x64_07eba1db5a2cfac83683bd43d727a3195a94b39b.msu",
        "win_vers": WindowsVersions.Windows10v1703,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows10.0-kb4577586-x86_9a3ade6ec24a2627276cbd7df0b39ece8ffee269.msu",
        "kb_name": "windows10.0-kb4577586-x86_9a3ade6ec24a2627276cbd7df0b39ece8ffee269.msu",
        "win_vers": WindowsVersions.Windows10v1703,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows8.1-kb4577586-x64_1c039dbdafaf12e9163e8d56cf578cc998306ecd.msu",
        "kb_name": "windows8.1-kb4577586-x64_1c039dbdafaf12e9163e8d56cf578cc998306ecd.msu",
        "win_vers": WindowsVersions.Windows81,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/d/msdownload/update/software/updt/2020/10/windows8.1-kb4577586-x86_d81dcb42a9b91d279c0b7510669bc0d556288fd3.msu",
        "kb_name": "windows8.1-kb4577586-x86_d81dcb42a9b91d279c0b7510669bc0d556288fd3.msu",
        "win_vers": WindowsVersions.Windows81,
        "arch": "x86",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows8-rt-kb4577586-x64_3dde08cdb47cb317ba47a9181fceee09951c614c.msu",
        "kb_name": "windows8-rt-kb4577586-x64_3dde08cdb47cb317ba47a9181fceee09951c614c.msu",
        "win_vers": WindowsVersions.Windows8,
        "arch": "x64",
    },
    {
        "dl_url": "http://download.windowsupdate.com/c/msdownload/update/software/updt/2020/10/windows8-rt-kb4577586-x86_2c105da7125b3d7a33edc8fd07c322261625bd3c.msu",
        "kb_name": "windows8-rt-kb4577586-x86_2c105da7125b3d7a33edc8fd07c322261625bd3c.msu",
        "win_vers": WindowsVersions.Windows8,
        "arch": "x86",
    },
]
bin_name_uninstaller = "uninstall_flash_player.exe"
silent_uninst_args = "-uninstall"
app_name = "Adobe Flash Player"
schtasks_to_delete = ["Adobe Flash Player Updater", "Adobe Flash Player PPAPI Notifier"]
services_to_delete = ["AdobeFlashPlayerUpdateSvc"]
processes_to_kill = ["firefox.exe", "iexplore.exe", "opera.exe", "chrome.exe"]


def install():
    # Initializing variables
    if iswin64():
        arch = "x64"
        app_dir = makepath(os.environ["SYSTEMROOT"], "SysWOW64", "Macromed", "Flash")
        app_macromed_dir = makepath(os.environ["SYSTEMROOT"], "SysWOW64", "Macromed")
    else:
        arch = "x86"
        app_dir = makepath(os.environ["SYSTEMROOT"], "System32", "Macromed", "Flash")
        app_macromed_dir = makepath(os.environ["SYSTEMROOT"], "System32", "Macromed")
    bin_app_path_w10 = makepath(app_dir, "FlashUtil_ActiveX.exe")

    # Uninstalling Adobe Flash Player
    if installed_softwares(name=app_name):
        killalltasks(processes_to_kill)
    run('"%s" %s' % (makepath(basedir, bin_name_uninstaller), silent_uninst_args))

    # Removing older version of Adobe Flash Player not handled by the uninstaller
    for uninstall in installed_softwares(app_name):
        print("Uninstalling: %s" % uninstall["name"])
        run_notfatal(uninstall_cmd(uninstall["key"]))

    # Deleting Scheduled Tasks
    for task in schtasks_to_delete:
        if task_exists(task):
            try:
                delete_task(task)
            except:
                print("Unable to delete the task: %s" % task)

    # Deleting Service
    for service in services_to_delete:
        if service_installed(service):
            try:
                service_delete(service)
            except:
                print("Unable to delete the service: %s" % service)

    # Uninstalling IE Adobe Flash Player on Windows 8+ with KB
    if windows_version() >= WindowsVersions.Windows8 and windows_version() < WindowsVersions.Windows10v21H1:
        if windows_version() < WindowsVersions.Windows10:
            winver = windows_version(members_count=2)
        else:
            winver = windows_version(members_count=3)
        for kb_dict in kbs_dict:
            if kb_dict["win_vers"] == winver and kb_dict["arch"] == arch:
                kb_file = kb_dict["kb_name"]
                kb_nb = kb_file.split("-")[1]
                break
        if kb_nb:
            with EnsureWUAUServRunning():
                if not is_kb_installed(kb_nb):
                    print("Installing {}".format(kb_file))
                    killalltasks("iexplore.exe")
                    run_notfatal('wusa.exe "{}" /quiet /norestart'.format(kb_file), accept_returncodes=[0, 3010, 2359302, -2145124329], timeout=3600)
                else:
                    print("{} already installed".format(kb_file))
        else:
            print("This Windows 10 version is not handled, you have to complete this package (complete the kbs_dict)")

    # Removing remaining files if acceptable
    time.sleep(30)
    try:
        if isdir(app_dir):
            for binary_w7 in glob.glob(makepath(app_dir, "*")):  # FlashPlayerPlugin_32_0_0_xxx.exe
                if binary_w7.startswith("FlashPlayerPlugin_") and binary_w7.endswith(".exe"):
                    bin_app_path_w7 = binary_w7
                    break
                else:
                    bin_app_path_w7 = makepath(app_dir, "filenotfound.exe")
            if isfile(bin_app_path_w10) or isfile(bin_app_path_w7):
                print("WARNING: %s seems not fully uninstalled" % app_name)
            else:
                run_notfatal("takeown /F %s /R" % app_dir)
                run_notfatal("icacls %s /reset /T" % app_dir)
                remove_tree(app_macromed_dir)
        else:
            if isdir(app_macromed_dir):
                remove_tree(app_macromed_dir)
    except:
        print("Unable to remove %s remaining files, you may need to manually fix the KB's installation" % app_name)


def session_setup():
    print("Removing: Flash Player files in user env")
    # Initializing variables
    user_app_dir_1 = makepath(application_data, "Adobe", "Flash Player")
    user_app_dir_2 = makepath(application_data, "Macromedia", "Flash Player")

    try:
        if isdir(user_app_dir_1):
            remove_tree(user_app_dir_1)
        if isdir(user_app_dir_2):
            remove_tree(user_app_dir_2)
    except:
        print("Unable to remove %s remaining files in user env")


def audit():
    # Initializing variables
    if iswin64():
        app_dir = makepath(os.environ["SYSTEMROOT"], "SysWOW64", "Macromed", "Flash")
    else:
        app_dir = makepath(os.environ["SYSTEMROOT"], "System32", "Macromed", "Flash")

    # Auditing the package
    if installed_softwares(name=app_name):
        print("ERROR: %s is still installed" % app_name)
        return "ERROR"

    if isdir(app_dir):
        print("WARNING: The directory %s remain, Flash may not be uninstalled for IE" % app_dir)
        return "WARNING"

    if not installed_softwares(name=app_name):
        print("OK: %s is not detected on the system" % app_name)
        return "OK"
